/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.FilenameFilter;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ToolException;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.ResUtil;

// JDK
import java.io.File;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;
import java.util.Properties;

/**
 * Class containing the version information
 */
public class KelpInfo {
    public static final String  ADDIN_ROOT = "addin.root";
    public static final String  ADDIN_HELP = "kelp.html";

    // not to be resourced
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private static final String VERSION = "5.0";                   // nores
    private static final String TOOLBOX_CLASS =
        "org.enhydra.tool.ToolBoxInfo";                             // nores

    //
    public static String getKelpVersion() {
        return KelpInfo.VERSION;
    }

    public static void verifyIDEClassPath() throws Exception {
        boolean valid = true;

        valid = KelpInfo.isClassPathComplete();
        StringBuffer message = new StringBuffer();

        if (!valid) {
            message.append(res.getString("MissingJars1"));
            message.append(res.getString("MissingJars2"));
            if (KelpInfo.isToolBoxInClassPath()) {
                if (!KelpInfo.isToolBoxVersionInSynch()) {
                    message.setLength(0);
                    message.append(res.getString("OutOfSync"));
                    message.append(ResUtil.format(res.getString("Kelp_version_0"),
                                                  KelpInfo.getKelpVersion()));
                    message.append('\n');
                    message.append(ResUtil.format(res.getString("ToolBox_version_0"),
                                                  ToolBoxInfo.getToolBoxVersion()));
                } else if (!ToolBoxInfo.isEnhydraInClassPath()) {
                    message.append(Constants.FILE_ENHYDRA_JAR);
                    message.append('\n');
                } else if (!ToolBoxInfo.isXMLCInClassPath()) {
                    message.append(Constants.FILE_XMLC_JAR);
                    message.append('\n');
                }
            } else {
                message.append(Constants.FILE_TOOLBOX_JAR);
                message.append('\n');
            }
            throw new Exception(message.toString());
        }
    }

    public static String getMessageBoxTitle() {
        StringBuffer buf = new StringBuffer();

        buf.append(Constants.KELP);
        buf.append(Constants.TAB1);
        buf.append(KelpInfo.VERSION);
        return buf.toString();
    }

    public static boolean isToolBoxInClassPath() {
        boolean inPath = true;

        try {
            Class testClass = Class.forName(TOOLBOX_CLASS);
        } catch (java.lang.ClassNotFoundException e) {
            inPath = false;
        }
        return inPath;
    }

    public static boolean isToolBoxVersionInSynch() {
        boolean inSynch = false;

        if (KelpInfo.isToolBoxInClassPath()) {
            if (ToolBoxInfo.getToolBoxVersion().equalsIgnoreCase(KelpInfo.getKelpVersion())) {
                inSynch = true;
            }
        }
        return inSynch;
    }

    public static boolean isClassPathComplete() {
        boolean complete = false;

        if (KelpInfo.isToolBoxInClassPath()) {
            if (ToolBoxInfo.isXMLCInClassPath()
                    && KelpInfo.isToolBoxVersionInSynch()) {
                complete = true;
            }
        }
        return complete;
    }

    public static String getAddinHelpURL() {
        String[] search = new String[0];

        return KelpInfo.getAddinHelpURL(search);
    }

    public static String getAddinHelpURL(String s) {
        String[] search = new String[1];

        if (s == null) {
          search = new String[0];
        } else {
          search[0] = s;
        }
        return KelpInfo.getAddinHelpURL(search);
    }

    public static String getAddinHelpURL(String[] search) {
        StringBuffer buf = new StringBuffer();
        File         file = null;

        buf.append(KelpInfo.getAddinRoot(search));
        buf.append("/doc/");
        buf.append(KelpInfo.ADDIN_HELP);
        file = new File(buf.toString());
        buf.setLength(0);
        try {
            buf.append(file.toURL().toExternalForm());
        } catch (MalformedURLException e) {
            e.printStackTrace(System.err);
        }
        return buf.toString();
    }

    private static String getAddinRoot(String[] search) {
        final String   FILE_ADDIN_JAR = "KelpAddin.jar";
        ArrayList      list = null;
        Properties     props = null;
        String         root = null;
        String[]       paths = new String[0];
        ClassLoader    loader = null;
        FilenameFilter filter = null;
        PathHandle     ph = null;
        KelpInfo       info = null;

        // find by class loader - will fail for forte
        info = new KelpInfo();
        loader = info.getClass().getClassLoader();
        paths = FileUtil.findJarPaths(FILE_ADDIN_JAR, loader);
        for (int i = 0; i < paths.length; i++) {
            ph = PathHandle.createPathHandle(paths[i]);
            ph = ph.getParent();
            if (ph.getParent() == null) {
                root = ph.getPath();
            } else {
                root = ph.getParent().getPath();
            }
            break;
        }

        // search
        if (root == null) {
            ph = PathHandle.createPathHandle(ToolBoxInfo.getEnhydraRoot());
            if (ph.getParent() != null) {
                list = new ArrayList(Arrays.asList(search));
                list.add(ph.getParent().getPath() + "/kelp4");
                list.trimToSize();
                search = new String[list.size()];
                search = (String[]) list.toArray(search);
                list.clear();
            }
            filter = new FilenameFilter();
            filter.setIncludeName(KelpInfo.ADDIN_HELP);
            filter.setDirectoryValid(false);
            for (int i = 0; i < search.length; i++) {
                ph = PathHandle.createPathHandle(FileUtil.findFirst(filter,
                        search[i]));
                if (ph.isFile()) {
                    ph = ph.getParent();
                    if (ph.getParent() != null) {
                        ph = ph.getParent();
                    }
                    root = ph.getPath();
                    break;
                }
            }
        }
        try {

            // if can't find root load prop, otherwise save
            props = ToolBoxInfo.loadProperties();
            if (root == null) {
                root = props.getProperty(ADDIN_ROOT);
            } else {
                props.setProperty(KelpInfo.ADDIN_ROOT, root);
                ToolBoxInfo.storeProperties(props);
            }
        } catch (ToolException e) {
            e.printStackTrace(System.err);
            root = null;
        }
        if (root == null) {
            root = "../kelp4";
        }
        return root;
    }

    /**
     * Prevent instantiation.
     */
    private KelpInfo() {}

}
