/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// AddinCore
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.swing.FileNodeSelectionPanel;

// ToolBox
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ToolException;

// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.PrintWriter;
import java.lang.ref.WeakReference;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.beans.*;

//
public class InputPanel extends JPanel implements Instructor, DeployStep {
    private JTabbedPane            tab;
    private GridBagLayout          layoutMain;
    private JPanel                 panelSelections;
    private GridBagLayout          layoutSelections;
    private FileNodeSelectionPanel selectionPanel;
    private JPanel                 panelOps;
    private GridBagLayout          layoutOps;
    private JCheckBox              checkInputFilter;
    private JPanel                 panelRoot;
    private GridBagLayout          layoutRoot;
    private JTextField             textInputRoot;
    private JButton                buttonRoot;
    private ReplacementTablePanel  replacementPanel;
    private JLabel                 labelRoot;
    private JCheckBox              checkEnable;
    private LocalButtonListener    buttonListener = null;
    private LocalCheckListener     checkListener = null;
    private WeakReference          projectRef = null;
    private boolean                ran = false;

    public InputPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // implements Instructor
    public String getTab() {
        return "Input";
    }

    // implements Instructor
    public String getTitle() {
        return "Input templates";
    }

    // implements Instructor
    public String getInstructions() {
        return "The deployer copies files from the input root to the deployment root. "
               + "The replacement table is used to search and replace tokens within the input templates during the copy.";
    }

    // implements DeployStep
    public void refresh() {
        if (getProject() != null) {
            checkEnable.setSelected(getProject().isDeployInput());
        }
        refreshEnable();
    }

    // implements DeployStep
    public void clearAll() {
        selectionPanel.clearAll();
        replacementPanel.clearAll();
        buttonRoot.removeActionListener(buttonListener);
        checkInputFilter.removeChangeListener(checkListener);
        checkEnable.removeChangeListener(checkListener);
        if (projectRef != null) {
            projectRef.clear();
        }
        removeAll();
        buttonListener = null;
        checkListener = null;
        projectRef = null;
    }

    // implements DeployStep
    public boolean isDataValid() {
        return DeployStepUtil.isDataValid(this);
    }

    // implements DeployStep
    public boolean isDataEqual(OtterProject project) {
        PathHandle projectPath = null;
        PathHandle panelPath = null;
        boolean    equal = true;

        projectRef = new WeakReference(project);
        if (project == null) {
            System.err.println("InnerPanel.isDataEqual() - project null");
            return true;
        } else if (checkInputFilter.isSelected()
                   != project.isDeployInputFilter()) {
            equal = false;
        }
        if (checkEnable.isSelected() != project.isDeployInput()) {
            equal = false;
        }
        if (equal) {
            projectPath =
                PathHandle.createPathHandle(project.getDeployInputPath());
            panelPath = PathHandle.createPathHandle(textInputRoot.getText());
            if (!projectPath.equals(panelPath)) {
                equal = false;
            }
        }
        if (equal) {
            equal = replacementPanel.isDataEqual();
        }

        // selectionPanel - always equal - realtime update
        return equal;
    }

    // implements DeployStep
    public void validateData() throws DataValidationException {
        PathHandle path = null;

        if (checkEnable.isSelected()) {
            path = PathHandle.createPathHandle(textInputRoot.getText());
            if (!path.getPath().equalsIgnoreCase(textInputRoot.getText())) {
                throw new DataValidationException("Invalid input root: "
                                                  + textInputRoot.getText());
            }
            if (!path.isDirectory()) {
                throw new DataValidationException("Invalid input root: "
                                                  + textInputRoot.getText());
            }
        }

        // selectionPanel - always valid
        // replacementTable - always valid
    }

    // implements DeployStep
    public boolean isBuilt(OtterProject project) {
        projectRef = new WeakReference(project);
        ran = DeployStepUtil.isBuilt(this, ran, project);
        return ran;
    }

    // implements DeployStep
    public boolean isSelectable(OtterProject project, DeployStep[] steps) {
        boolean able = false;

        projectRef = new WeakReference(project);
        for (int i = 0; i < steps.length; i++) {
            if (steps[i] instanceof GeneralPanel) {
                able = steps[i].isBuilt(project);
                break;
            }
        }
        return able;
    }

    // implements DeployStep
    public void build(OtterProject project, WriteListener listener)
            throws DataValidationException, ToolException {
        InputBuilder builder = null;

        projectRef = new WeakReference(project);
        if (!isBuilt(project)) {
            write(project);
            builder = new InputBuilder(listener);
            builder.setProject(project);
            builder.setOwner(this);
            builder.setEcho(true);
            builder.buildInCurrentThread();
            ran = true;
        }
    }

    // implements DeployStep
    public void read(OtterProject project) {
        PathHandle path = null;

        projectRef = new WeakReference(project);
        path = PathHandle.createPathHandle(project.getDeployInputPath());
        if (path.isDirectory()) {
            textInputRoot.setText(path.getPath());
        } else {
            textInputRoot.setText(new String());
        }
        textInputRoot.setToolTipText(textInputRoot.getText());
        selectionPanel.setNodes(project.getAllInput());
        replacementPanel.setNode(project);
        replacementPanel.readProperties();
        checkInputFilter.setSelected(project.isDeployInputFilter());
        checkEnable.setSelected(project.isDeployInput());
        refreshEnable();
    }

    // implements DeployStep
    public void write(OtterProject project) throws DataValidationException {
        projectRef = new WeakReference(project);
        validateData();
        if (!isDataEqual(project)) {
            project.setDeployInput(checkEnable.isSelected());
            project.setDeployInputFilter(checkInputFilter.isSelected());
            project.setDeployInputPath(PathHandle.createPathString(textInputRoot.getText()));
        }
        replacementPanel.writeProperties();

        // selection panel - uses realtime write
    }

    //
    //
    private void refreshEnable() {
        boolean enable = checkEnable.isSelected();

        buttonRoot.setEnabled(enable);
        checkInputFilter.setEnabled(enable);
        selectionPanel.setEnabled(enable);
        replacementPanel.setEnabled(enable);
        for (int i = 0; i < tab.getTabCount(); i++) {
            tab.setEnabledAt(i, enable);
        }
    }

    private void refreshSelections() {
        if (getProject() != null) {
            getProject().setDeployInputFilter(checkInputFilter.isSelected());
            getProject().setDeployInputPath(textInputRoot.getText());
            selectionPanel.setNodes(getProject().getAllInput());
        }
    }

    private void chooseRootDirectory() {
        File       choice = null;
        PathHandle path = null;

        choice =
            SwingUtil.getDirectoryChoice(this, textInputRoot.getText(),
                                         "Select input template directory");
        path = PathHandle.createPathHandle(choice);
        if (path.isDirectory()) {
            textInputRoot.setText(path.getPath());
            textInputRoot.setToolTipText(textInputRoot.getText());
            refreshSelections();
        }
    }

    private OtterProject getProject() {
        OtterProject project = null;

        if (projectRef != null) {
            project = (OtterProject) projectRef.get();
        }
        return project;
    }

    private void pmInit() {
        String[] text = {
            "Input available", "Input selected"
        };

        buttonListener = new LocalButtonListener();
        buttonRoot.addActionListener(buttonListener);
        selectionPanel.setSelectText(text);
        checkListener = new LocalCheckListener();
        checkInputFilter.addChangeListener(checkListener);
        checkEnable.addChangeListener(checkListener);
    }

    private void jbInit() throws Exception {
        tab = (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                              JTabbedPane.class.getName());
        panelSelections =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        selectionPanel =
            (FileNodeSelectionPanel) Beans.instantiate(getClass().getClassLoader(),
                FileNodeSelectionPanel.class.getName());
        panelOps = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        layoutOps =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        checkInputFilter =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        panelRoot = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        layoutRoot =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textInputRoot =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonRoot = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        layoutSelections =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        replacementPanel =
            (ReplacementTablePanel) Beans.instantiate(getClass().getClassLoader(),
                ReplacementTablePanel.class.getName());
        labelRoot = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        checkEnable =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelOps.setLayout(layoutOps);
        checkInputFilter.setText("Input templates (.in) only");
        panelRoot.setLayout(layoutRoot);
        textInputRoot.setEnabled(false);
        textInputRoot.setEditable(false);
        textInputRoot.setText("INTPUTROOT");
        buttonRoot.setText("...");
        labelRoot.setText("Input root:");
        checkEnable.setSelected(true);
        checkEnable.setText("Enable input deployment");
        panelOps.add(checkInputFilter,
                     new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                            GridBagConstraints.WEST,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 10, 0, 5), 0, 0));
        panelOps.add(panelRoot,
                     new GridBagConstraints(0, 1, 2, 1, 0.1, 0.0,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(0, 5, 0, 5), 0, 0));
        panelRoot.add(textInputRoot,
                      new GridBagConstraints(1, 0, 1, 1, 0.1, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.HORIZONTAL,
                                             new Insets(2, 5, 2, 5), 0, 0));
        panelRoot.add(buttonRoot,
                      new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.NONE,
                                             new Insets(1, 5, 1, 5), 0, 0));
        panelRoot.add(labelRoot,
                      new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
                                             GridBagConstraints.WEST,
                                             GridBagConstraints.NONE,
                                             new Insets(2, 5, 2, 5), 0, 0));
        panelSelections.setLayout(layoutSelections);
        panelSelections.add(selectionPanel,
                            new GridBagConstraints(0, 0, 1, 1, 0.2, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.BOTH,
                                                   new Insets(0, 5, 0, 5), 0,
                                                   0));
        panelSelections.add(panelOps,
                            new GridBagConstraints(0, 1, 1, 1, 0.1, 0.0,
                                                   GridBagConstraints.CENTER,
                                                   GridBagConstraints.HORIZONTAL,
                                                   new Insets(0, 0, 0, 0), 0,
                                                   0));
        tab.add(panelSelections, "Selections");
        tab.add(replacementPanel, "Replacements");
        this.setLayout(layoutMain);
        this.add(checkEnable,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(2, 10, 2, 10), 0, 0));
        this.add(tab,
                 new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(0, 0, 0, 0), -198, 15));
    }

    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonRoot) {
                chooseRootDirectory();
            }
        }

    }
    private class LocalCheckListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            Object source = e.getSource();
            if (source == checkInputFilter) {
                refreshSelections();
            } else if (source == checkEnable) {
               if (getProject() != null) {
                  getProject().setDeployInput(checkEnable.isSelected());
               }
                refreshEnable();
            }
        }

    }
}
