/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.deployer;

// AddinCore
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.event.ExtensionChangeListener;
import org.enhydra.kelp.common.event.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.DataValidationException;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ExtensionFilter;

// JDK
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.*;
import java.io.File;
import java.io.PrintWriter;
import java.lang.ref.WeakReference;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.border.*;

//
public class RunPanel extends JPanel implements Instructor, DeployStep {
    private GridBagLayout       layoutMain;
    private JRadioButton        radioAuto;
    private JRadioButton        radioIDE;
    private JLabel              labelAuto;
    private GridBagLayout       layoutAuto;
    private JTextField          textAuto;
    private JButton             buttonAuto;
    private JLabel              labelBoot;
    private JTextField          textBoot;
    private JButton             buttonBoot;
    private GridBagLayout       layoutBoot;
    private JLabel              labelConf;
    private JTextField          textConf;
    private GridBagLayout       layoutConf;
    private JCheckBox           checkStartupJava;
    private JLabel              labelAutoFile;
    private JTextField          textAutoFile;
    private JButton             buttonAutoFile;
    private JCheckBox           checkEnable;
    private LocalCheckListener  checkListener = null;
    private LocalRadioListener  radioListener = null;
    private LocalButtonListener buttonListener = null;
    private WeakReference       projectRef = null;
    private boolean             ran = false;

    //
    public RunPanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    // implements Instructor
    public String getTab() {
        return "Run";
    }

    // implements Instructor
    public String getTitle() {
        return "Run configuration";
    }

    // implements Instructor
    public String getInstructions() {
        return "The deployer can deploy archives to a running locally server or "
               + "setup your project so that you can launch the server from your IDE.";
    }

    // implements DeployStep
    public void refresh() {
        if (getProject() != null) {
            if (getProject().getDeployType() == OtterProject.TYPE_SERVICE) {
                radioAuto.setSelected(true);
                radioIDE.setEnabled(false);
                radioIDE.setSelected(false);
            }
        }
        radioRefresh();
    }

    // implements DeployStep
    public void clearAll() {
        radioAuto.removeItemListener(radioListener);
        radioIDE.removeItemListener(radioListener);
        buttonAuto.removeActionListener(buttonListener);
        buttonBoot.removeActionListener(buttonListener);
        checkEnable.removeChangeListener(checkListener);
        removeAll();
        if (projectRef != null) {
            projectRef.clear();
        }
        projectRef = null;
        checkListener = null;
        radioListener = null;
        buttonListener = null;
    }

    // implements DeployStep
    public boolean isDataValid() {
        return DeployStepUtil.isDataValid(this);
    }

    // implements DeployStep
    public boolean isDataEqual(OtterProject project) {
        boolean    equal = true;
        PathHandle projectPath = null;
        PathHandle panelPath = null;

        projectRef = new WeakReference(project);
        if (!project.isDeployStartupJavaReadOnly()) {
            if (project.isDeployStartupJava()
                    != checkStartupJava.isSelected()) {
                equal = false;
            }
        }
        if (equal) {
            if (project.isDeployRun() != checkEnable.isSelected()) {
                equal = false;
            }
        }
        if (equal) {
            projectPath =
                PathHandle.createPathHandle(project.getDeployBootstrapPath());
            if (radioAuto.isSelected()) {
                panelPath = PathHandle.createPathHandle(textAuto.getText());
            } else {
                panelPath = PathHandle.createPathHandle(textBoot.getText());
            }
            if (!projectPath.equals(panelPath)) {
                equal = false;
            }
        }
        if (equal) {
            projectPath =
                PathHandle.createPathHandle(project.getAutoDeployFilePath());
            panelPath = PathHandle.createPathHandle(textAutoFile.getText());
            if (!projectPath.equals(panelPath)) {
                equal = false;
            }
        }
        return equal;
    }

    // implements DeployStep
    public void validateData() throws DataValidationException {
        PathHandle path = null;

        if (checkEnable.isSelected()) {
            if (radioAuto.isSelected()) {
                path = PathHandle.createPathHandle(textAuto.getText());
                if (!path.isDirectory()) {
                    throw new DataValidationException("Invalid auto deployment directory: "
                                                      + path.getPath());
                }
            } else {
                path = PathHandle.createPathHandle(textBoot.getText());
                if (!path.isFile()) {
                    throw new DataValidationException("Invalid server configuration file: "
                                                      + path.getPath());
                }
            }
        }
    }

    // implements DeployStep
    public boolean isBuilt(OtterProject project) {
        projectRef = new WeakReference(project);
        ran = DeployStepUtil.isBuilt(this, ran, project);
        return ran;
    }

    // implements DeployStep
    public boolean isSelectable(OtterProject project, DeployStep[] steps) {
        boolean able = false;
        boolean generalBuilt = false;
        boolean inputBuilt = false;
        boolean contentBuilt = false;
        boolean archiveBuilt = false;

        projectRef = new WeakReference(project);
        for (int i = 0; i < steps.length; i++) {
            if (steps[i] instanceof GeneralPanel) {
                generalBuilt = steps[i].isBuilt(project);
            } else if (steps[i] instanceof InputPanel) {
                inputBuilt = steps[i].isBuilt(project);
            } else if (steps[i] instanceof ContentPanel) {
                contentBuilt = steps[i].isBuilt(project);
            } else if (steps[i] instanceof ArchivePanel) {
                archiveBuilt = steps[i].isBuilt(project);
            }
        }
        able = generalBuilt && inputBuilt && contentBuilt && archiveBuilt;
        return able;
    }

    // implements DeployStep
    public void build(OtterProject project,
                      WriteListener listener) throws DataValidationException {
        RunBuilder builder = null;

        projectRef = new WeakReference(project);
        if (!isBuilt(project)) {
            write(project);
            builder = new RunBuilder(listener);
            builder.setProject(project);
            builder.setEcho(true);
            builder.buildInCurrentThread();
            ran = true;
        }
    }

    // implements DeployStep
    public void read(OtterProject project) {
        PathHandle path = null;

        projectRef = new WeakReference(project);
        refresh();
        path = PathHandle.createPathHandle(project.getDeployBootstrapPath());
        checkEnable.setSelected(project.isDeployRun());
        if (path.isDirectory()) {
            textAuto.setText(path.getPath());
            textBoot.setText(new String());
            radioAuto.setSelected(true);
            radioIDE.setSelected(false);
        } else if (path.hasExtension("conf")) {
            textAuto.setText(new String());
            textBoot.setText(path.getPath());
            radioAuto.setSelected(false);
            radioIDE.setSelected(true);
        } else {
            textAuto.setText(new String());
            textBoot.setText(new String());
            radioAuto.setSelected(false);
            radioIDE.setSelected(true);
        }
        textAutoFile.setText(project.getAutoDeployFilePath());
        textAutoFile.setToolTipText(textAutoFile.getText());
        textAuto.setToolTipText(textAuto.getText());
        textBoot.setToolTipText(textBoot.getText());
        textConf.setText(PathUtil.getDeployConfPath(project.getDeployRootPath()));
        textConf.setToolTipText(textConf.getText());
        if (project.isDeployStartupJavaReadOnly()) {
            checkStartupJava.setVisible(false);
        } else {
            checkStartupJava.setSelected(project.isDeployStartupJava());
        }
    }

    // implements DeployStep
    public void write(OtterProject project) throws DataValidationException {
        PathHandle path = null;

        projectRef = new WeakReference(project);
        validateData();
        if (!isDataEqual(project)) {
            if (radioAuto.isSelected()) {
                path = PathHandle.createPathHandle(textAuto.getText());
            } else {
                path = PathHandle.createPathHandle(textBoot.getText());
            }
            project.setDeployBootstrapPath(path.getPath());
            project.setAutoDeployFile(textAutoFile.getText());
            if (!project.isDeployStartupJavaReadOnly()) {
                project.setDeployStartupJava(checkStartupJava.isSelected());
            }
            project.setDeployRun(checkEnable.isSelected());
        }
    }

    //
    private void chooseDirectory(JTextField text, String title) {
        File       choice = null;
        PathHandle path = null;

        choice = SwingUtil.getDirectoryChoice(this, text.getText(), title);
        path = PathHandle.createPathHandle(choice);
        if (path.isDirectory()) {
            text.setText(path.getPath());
            text.setToolTipText(text.getText());
        }
    }

    private void chooseBootFile() {
        File            choice = null;
        PathHandle      path = null;
        ExtensionFilter filter = null;

        filter = new ExtensionFilter();
        filter.setDescriptionTitle("Server Configuration");
        filter.addExtension("conf");
        choice =
            SwingUtil.getFileChoice(this, textBoot.getText(), filter,
                                    "Select a server bootstrap configuration file");
        path = PathHandle.createPathHandle(choice);
        if (path.isFile()) {
            textBoot.setText(path.getPath());
            textBoot.setToolTipText(textBoot.getText());
        }
    }

    private void chooseAutoFile() {
        File            choice = null;
        PathHandle      path = null;
        ExtensionFilter filter = null;
        int             type = OtterProject.TYPE_UNKNOWN;

        if (getProject() != null) {
            type = getProject().getDeployType();
        }
        filter = new ExtensionFilter();
        switch (type) {
        case OtterProject.TYPE_EN3APP:
            filter.setDescriptionTitle("Enhydra App Configuration");
            filter.addExtension("conf");
            break;
        case OtterProject.TYPE_WEBAPP:
            filter.setDescriptionTitle("Application Archive");
            filter.addExtension("war");
            filter.addExtension("ear");
            break;
        case OtterProject.TYPE_SERVICE:
            filter.setDescriptionTitle("Service Archive");
            filter.addExtension("jar");
            break;
        default:
            filter.setDescriptionTitle("Deployment File");
            break;
        }
        choice = SwingUtil.getFileChoice(this, textAutoFile.getText(),
                                         filter,
                                         "Select file to copy to server");
        path = PathHandle.createPathHandle(choice);
        if (path.isFile()) {
            textAutoFile.setText(path.getPath());
            textAutoFile.setToolTipText(textAutoFile.getText());
        }
    }

    private void radioRefresh() {
        boolean    auto = radioAuto.isSelected();
        PathHandle ph = null;
        String     def = null;
        int        type = OtterProject.TYPE_UNKNOWN;

        buttonBoot.setEnabled(!auto);
        labelBoot.setEnabled(!auto);
        labelConf.setEnabled(!auto);
        checkStartupJava.setEnabled(!auto);
        buttonAuto.setEnabled(auto);
        buttonAutoFile.setEnabled(auto);
        labelAuto.setEnabled(auto);
        labelAutoFile.setEnabled(auto);
        if (auto) {
            if (getProject() != null) {
                type = getProject().getDeployType();
            }
            textBoot.setBackground(SystemColor.control);
            textConf.setBackground(SystemColor.control);
            textAuto.setBackground(SystemColor.info);
            textAutoFile.setBackground(SystemColor.info);
            if (textAuto.getText().trim().length() == 0) {
                def = PathUtil.getAutoDeployPath(type);
                ph = PathHandle.createPathHandle(def);
                if (ph.isDirectory()) {
                    textAuto.setText(ph.getPath());
                }
            }
        } else {
            textBoot.setBackground(SystemColor.info);
            textConf.setBackground(SystemColor.info);
            textAuto.setBackground(SystemColor.control);
            textAutoFile.setBackground(SystemColor.control);
            if (textBoot.getText().trim().length() == 0) {
                if (getProject() != null) {
                    def =
                        PathUtil.getDefaultDeployBootstrapPath(getProject());
                    ph = PathHandle.createPathHandle(def);
                    if (ph.isFile()) {
                        textBoot.setText(ph.getPath());
                    }
                }
            }
        }
    }

    private OtterProject getProject() {
        OtterProject project = null;

        if (projectRef != null) {
            project = (OtterProject) projectRef.get();
        }
        return project;
    }

    private void pmInit() {
        ButtonGroup group = null;

        group = new ButtonGroup();
        group.add(radioAuto);
        group.add(radioIDE);

        //
        radioListener = new LocalRadioListener();
        radioAuto.addItemListener(radioListener);
        radioIDE.addItemListener(radioListener);

        //
        buttonListener = new LocalButtonListener();
        buttonAuto.addActionListener(buttonListener);
        buttonAutoFile.addActionListener(buttonListener);
        buttonBoot.addActionListener(buttonListener);

        //
        if (ToolBoxInfo.isEnhydra3()) {
            radioAuto.setEnabled(false);
            radioAuto.setSelected(false);
        }
        checkListener = new LocalCheckListener();
        checkEnable.addChangeListener(checkListener);
    }

    private void refreshEnable() {
        if (checkEnable.isSelected()) {
            radioAuto.setEnabled(true);
            if (getProject() == null) {
                radioIDE.setEnabled(true);
            } else if (getProject().getDeployType()
                       == OtterProject.TYPE_SERVICE) {
                radioIDE.setEnabled(false);
            } else {
                radioIDE.setEnabled(true);
            }
            radioRefresh();
        } else {
            buttonAuto.setEnabled(false);
            buttonAutoFile.setEnabled(false);
            buttonBoot.setEnabled(false);
            checkStartupJava.setEnabled(false);
            labelAuto.setEnabled(false);
            labelAutoFile.setEnabled(false);
            labelBoot.setEnabled(false);
            labelConf.setEnabled(false);
            radioAuto.setEnabled(false);
            radioIDE.setEnabled(false);
            textAuto.setBackground(SystemColor.control);
            textAutoFile.setBackground(SystemColor.control);
            textBoot.setBackground(SystemColor.control);
            textConf.setBackground(SystemColor.control);
        }
    }

    private void jbInit() throws Exception {
        checkStartupJava =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        labelConf = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        textConf = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        layoutConf =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        labelBoot = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        textBoot = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonBoot = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        layoutBoot =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioIDE =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        labelAuto = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                               JLabel.class.getName());
        layoutAuto =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        textAuto = (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                                  JTextField.class.getName());
        buttonAuto = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioAuto =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        labelAutoFile =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        textAutoFile =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonAutoFile =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        checkEnable =
            (JCheckBox) Beans.instantiate(getClass().getClassLoader(),
                                          JCheckBox.class.getName());
        radioAuto.setText("Auto-deploy to locally running server");
        radioIDE.setText("Configure project for starting server from IDE");
        textAuto.setEnabled(false);
        textAuto.setEditable(false);
        textAuto.setText("AUTODEPLOY DIRECTORY");
        buttonAuto.setText("...");
        textBoot.setEnabled(false);
        textBoot.setEditable(false);
        textBoot.setText("BOOT CONF");
        buttonBoot.setToolTipText("");
        buttonBoot.setText("...");
        textConf.setEnabled(false);
        textConf.setEditable(false);
        textConf.setText("TEXT CONF");
        labelAuto.setText("Deploy directory:");
        labelBoot.setText("Server configuration:");

        //
        labelConf.setText("Configuration root:");
//        checkStartupJava.setText("Create StartServer.java");
        labelAutoFile.setText("Deploy file:");
        textAutoFile.setEnabled(false);
        textAutoFile.setEditable(false);
        textAutoFile.setText("AUTODEPLOY FILE");
        buttonAutoFile.setText("...");
        checkEnable.setSelected(true);
        checkEnable.setText("Enable run configuration");
        checkEnable.setActionCommand("checkEnable");
        this.setLayout(layoutMain);
        this.add(labelConf,
                 new GridBagConstraints(0, 6, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 35, 3, 5), 0, 0));
        this.add(textConf,
                 new GridBagConstraints(1, 6, 2, 1, 0.6, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 5, 3, 5), 0, 0));
        this.add(labelBoot,
                 new GridBagConstraints(0, 5, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 35, 3, 5), 0, 0));
        this.add(textBoot,
                 new GridBagConstraints(1, 5, 1, 1, 0.6, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 5, 3, 5), 0, 0));
        this.add(buttonBoot,
                 new GridBagConstraints(2, 5, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(3, 5, 3, 5), 0, 0));
        this.add(radioAuto,
                 new GridBagConstraints(0, 1, 3, 1, 0.1, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 20, 0, 5), 0, 0));
        this.add(radioIDE,
                 new GridBagConstraints(0, 4, 3, 1, 0.1, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(10, 20, 0, 5), 0, 0));
        this.add(labelAuto,
                 new GridBagConstraints(0, 2, 1, 1, 0.1, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 35, 3, 5), 0, 0));
        this.add(textAuto,
                 new GridBagConstraints(1, 2, 1, 1, 0.6, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 5, 3, 5), 0, 0));
        this.add(buttonAuto,
                 new GridBagConstraints(2, 2, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(3, 5, 3, 5), 0, 0));
/*        this.add(checkStartupJava,
                 new GridBagConstraints(0, 7, 3, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 35, 3, 5), 0, 0));
*/        this.add(labelAutoFile,
                 new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 35, 3, 0), 0, 0));
        this.add(textAutoFile,
                 new GridBagConstraints(1, 3, 1, 1, 0.6, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(3, 5, 3, 5), 0, 0));
        this.add(buttonAutoFile,
                 new GridBagConstraints(2, 3, 1, 1, 0.0, 0.0,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.NONE,
                                        new Insets(3, 5, 3, 5), 0, 0));
        this.add(checkEnable,
                 new GridBagConstraints(0, 0, 3, 1, 0.0, 0.0,
                                        GridBagConstraints.WEST,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(0, 10, 0, 10), 0, 0));
    }

    //
    private class LocalRadioListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            if (e.getStateChange() == ItemEvent.SELECTED) {
                radioRefresh();
            }
        }

    }

    //
    private class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonAuto) {
                chooseDirectory(textAuto, "Select the auto-deploy directory");
            } else if (source == buttonAutoFile) {
                chooseAutoFile();
            } else if (source == buttonBoot) {
                chooseBootFile();
            }
        }

    }

    //
    private class LocalCheckListener implements ChangeListener {
        public void stateChanged(ChangeEvent e) {
            Object source = e.getSource();

            if (source == checkEnable) {
                refreshEnable();
            }
        }

    }
}
