/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.xmlc;

// Kelp imports
import org.enhydra.kelp.common.bridge.Generator;
import org.enhydra.kelp.common.bridge.Parser;
import org.enhydra.kelp.common.bridge.PrintInfo;
import org.enhydra.kelp.common.bridge.XMLCConnectionFactory;
import org.enhydra.kelp.common.node.OtterXMLCNode;

// Enhydra imports
import org.enhydra.xml.xmlc.commands.xmlc.*;
import org.enhydra.xml.xmlc.commands.options.*;
import org.enhydra.xml.xmlc.compiler.EditDOM;
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.parsers.*;
import org.enhydra.xml.xmlc.XMLCUtil;
import org.enhydra.xml.xmlc.XMLCVersion;
import org.enhydra.xml.xmlc.XMLCException;
import org.w3c.dom.*;
import org.xml.sax.SAXException;

// Standard imports
import java.io.*;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.util.ResourceBundle;

/**
 * Class derived from the XMLC compiler program for use in this wizard.
 * TODO: Mark may change some things that will require changes to this class.
 */
public class XMLCRunner {

    // string not to be resourced
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private final String  JAVA_LANG = "java.lang.";                 // nores

    /*
     * Object containing the parsed standard options.
     */
    private OtterXMLCNode node;

    /*
     * Document being compiled.
     */
    private Document      document;

    /*
     * Trace and verbose output.
     */
    private PrintWriter   traceWriter = new PrintWriter(System.err, true);

    /*
     * XML DOM management object.
     */
    private XMLCDocument  xmlcDoc;

    /**
     * Constructor declaration
     *
     *
     * @see
     */
    public XMLCRunner() {}

    /**
     * Method declaration
     *
     *
     * @param tw
     *
     * @see
     */
    public void setTraceWriter(PrintWriter tw) {
        traceWriter = tw;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public PrintWriter getTraceWriter() {
        return traceWriter;
    }

    /*
     * Print out a stack trace.  This works around bugs in SAXException that
     * cause printing a stack trace to generate a null pointer exception.  We
     * can't just fix this in the OpenXML source (we did), because other
     * parsers include that same broken class.
     */

    /**
     * Method declaration
     *
     *
     * @param except
     *
     * @see
     */
    private void printStackTrace(Throwable except) {

        // FIXME: needed???
        if (except instanceof SAXException) {
            Exception except2 = ((SAXException) except).getException();

            if (except2 != null) {
                except2.printStackTrace();
                return;
            }
        }
        except.printStackTrace();
    }

    /*
     * Handle an exception, printing the approriate error
     * message.
     */

    /**
     * Method declaration
     *
     *
     * @param except
     *
     * @see
     */
    public void printException(Throwable except) {
        except.printStackTrace();

        // Must check that options have been parsed before using them.
        if (except instanceof XMLCException) {
            System.err.println(res.getString("Error_") + except.getMessage());
            if ((node.getMetaDataHandler() != null)
                    && (node.getMetaDataHandler().getVerbose())) {
                printStackTrace(except);
            }

            // If this is result of a java.lang exception, print stack.
            Throwable cause = ((XMLCException) except).getCause();

            if ((cause != null)
                    && cause.getClass().getName().startsWith(JAVA_LANG)) {
                printStackTrace(cause);
            }
        } else if ((except instanceof FileNotFoundException)
                   || (except instanceof IOException)) {
            System.err.println(res.getString("Error_")
                               + except.getClass().getName()
                               + res.getString("LineEnd")
                               + except.getMessage());
            if ((node.getMetaDataHandler() != null)
                    && (node.getMetaDataHandler().getVerbose())) {
                printStackTrace(except);
            }
        } else {
            System.err.println(res.getString("Error_")
                               + except.getClass().getName()
                               + res.getString("LineEnd")
                               + except.getMessage());
            printStackTrace(except);
        }
        System.exit(1);
    }

    /*
     * Parse the page into the DOM and perform various checks and edits.
     */

    /**
     * Method declaration
     *
     *
     * @throws FileNotFoundException
     * @throws IOException
     * @throws SAXException
     * @throws XMLCException
     *
     * @see
     */

    /*
     * Parse the page into the DOM and perform various checks and edits.
     */
    private void parsePage()
            throws Throwable {
        if (node.getMetaDataHandler().getVerbose()) {
            traceWriter.println(res.getString("_parsing")
                                + node.getMetaDataHandler().getInputDocument());
        }
        Parser parser = null;

        parser =
            XMLCConnectionFactory.createParser(traceWriter,
                                               node.getMetaDataHandler());
        xmlcDoc = (XMLCDocument) parser.parse();
        document = xmlcDoc.getDocument();
        if (node.getMetaDataHandler().getPrintDocumentInfo()) {
            if (node.getMetaDataHandler().getVerbose()) {
                traceWriter.println(res.getString("_printing_document"));
            }
            PrintInfo info = XMLCConnectionFactory.createPrintInfo(document,
                    xmlcDoc);

            info.printInfo(traceWriter);
        }
        EditDOM editDOM = null;

        editDOM =
            XMLCConnectionFactory.createEditDOM(node.getMetaDataHandler());
        editDOM.edit(xmlcDoc);
        if (node.getMetaDataHandler().getPrintDOM()) {
            if (node.getMetaDataHandler().getVerbose()) {
                traceWriter.println(res.getString("_dumping_document"));
            }
            XMLCUtil.printNode(res.getString("DOM_hierarchy"), document,
                               XMLCUtil.PRINT_ALL, traceWriter);
        }
    }

    /*
     * Generate the Java source files.
     */

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    private void generateJavaSource() throws XMLCException, IOException {
        PrintWriter accessorWriter = null;
        PrintWriter verboseWriter = null;
        Generator   codeGen = null;

        if (node.getMetaDataHandler().getPrintAccessorInfo()) {
            accessorWriter = traceWriter;
        }
        if (node.getMetaDataHandler().getVerbose()) {
            traceWriter.println(res.getString("_generating_code"));
            verboseWriter = traceWriter;
        }
        codeGen =
            XMLCConnectionFactory.createGenerator(node.getMetaDataHandler(),
                                                  xmlcDoc, accessorWriter);
        codeGen.generateJavaSource(verboseWriter);
    }

    /*
     * Generate method information without generating code.
     */

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    private void generateJavaSourceInfo() throws XMLCException, IOException {
        if (node.getMetaDataHandler().getVerbose()) {
            traceWriter.println(res.getString("_generating_code_info"));
        }
        Generator codeGen = XMLCConnectionFactory.createGenerator(node.getMetaDataHandler(),
                xmlcDoc,
                (node.getMetaDataHandler().getPrintAccessorInfo()
                 ? traceWriter : null));

        codeGen.generateJavaSource(node.getMetaDataHandler().getVerbose()
                                   ? traceWriter : null);
    }

    /**
     * Write the DOM to a file.
     */
    private void writeDOM() throws IOException {
        File   outputFile =
            new File(node.getMetaDataHandler().getDocumentOutput());
        String dir = outputFile.getParent();

        if (dir != null) {
            new File(dir).mkdirs();
        }
        PrintWriter domOut =
            new PrintWriter(new BufferedOutputStream(new FileOutputStream(node.getMetaDataHandler().getDocumentOutput())));

        try {
            domOut.println(xmlcDoc.toDocument());
        }
        finally {
            domOut.close();
        }
    }

    /*
     * Generate the Jave source and Compile the page.
     */

    /**
     * Method declaration
     *
     *
     * @exception FileNotFoundException
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    private void compilePage()
            throws XMLCException, FileNotFoundException, IOException {
        generateJavaSource();

        // do not compile Java and always check for recomp.
        if (node.getMetaDataHandler().getRecompilation()) {
            File op = node.getOptionFileForRecomp();

            if (!op.getParentFile().exists()) {
                op.getParentFile().mkdirs();
            }
            node.getMetaDataHandler().save(node.getOptionFileForRecomp());

            // node.getMetaDataHandler().getMetaData().getMetaDataDocument().serialize();
        }

        // do not compile Java
        // never delete source.
    }

    /**
     * Print the XMLC version number.
     */
    private void printVersion() {
        System.out.println(res.getString("Enhydra_XMLC_version")
                           + XMLCVersion.VERSION);
        System.out.println(res.getString("See_http_www_enhydra"));
    }

    /*
     * Parse arguments and compile the page.
     */

    /**
     * Method declaration
     *
     *
     * @throws FileNotFoundException
     * @throws IOException
     * @throws SAXException
     * @throws XMLCException
     *
     * @see
     */
    public void compile()
            throws Throwable {

        // Setup and document parsing.
        node.initProjectOptions();
        if (node.getMetaDataHandler().getPrintVersion()) {
            printVersion();
            if (node.getMetaDataHandler().getInputDocument() == null) {
                return;
            }
        }
        parsePage();
        if (node.getMetaDataHandler().getDocumentOutput() == null) {
            compilePage();
        } else {
            if (node.getMetaDataHandler().getPrintAccessorInfo()) {
                generateJavaSourceInfo();
            }
            writeDOM();
        }
        if (node.getMetaDataHandler().getVerbose()) {
            traceWriter.println(res.getString("_completed"));
        }
    }

    /**
     * Method declaration
     *
     *
     * @param cn
     *
     * @see
     */
    public void setNode(OtterXMLCNode n) {
        node = n;
    }

}
