/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive;

// ToolBox
import org.enhydra.tool.archive.xml.DescriptorHandler;
import org.enhydra.tool.archive.xml.EjbDescriptorHandler;
import org.enhydra.tool.common.PathHandle;

// JDK
import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

//
public class Descriptor {
    public static final String EJB = "ejb-jar";
    public static final String JONAS_EJB = "jonas-ejb-jar";
    public static final String WEB = "web";
    public static final String ENHYDRA_WEB = "enhydra-web";
    public static final String ENHYDRA_SERVICE = "enhydra-service";

    //
    private boolean            required = false;
    private String             type = new String();
    private String             path = new String();

    public static String getArchivePath(String type) {
        StringBuffer buf = new StringBuffer();

        if (type.endsWith("web")) {
            buf.append("WEB-INF/");
        } else {
            buf.append("META-INF/");
        }
        buf.append(type);
        buf.append(".xml");
        return buf.toString();
    }

    public Descriptor(boolean r, String t) {
        required = r;
        type = t;
    }

    public Descriptor(String p) {
      PathHandle ph = null;
      setPath(p);
      required = false;
      ph = PathHandle.createPathHandle(getPath());
      type = ph.getFile().getName();
      type = type.substring(0, type.indexOf('.'));
    }

    public boolean isRequired() {
        return required;
    }

    public String getType() {
        return type;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String p) {
        if (p == null) {
            path = new String();
        } else {
            PathHandle ph = PathHandle.createPathHandle(p);

            path = ph.getPath();
        }
    }

    public InputStream getInputStream() throws ArchiveException {
        DescriptorHandler dh = null;
        InputStream       stream = null;

        if (getType().equals(Descriptor.EJB)) {
            dh = new EjbDescriptorHandler();
        }
        if (dh != null) {
            dh.setSource(getPath());
            dh.prep();
            stream = dh.getInputStream();
        } else {
            FileInputStream     fileStream = null;
            BufferedInputStream bufStream = null;

            try {
                fileStream = new FileInputStream(getPath());
                bufStream = new BufferedInputStream(fileStream);
                stream = bufStream;
            } catch (FileNotFoundException e) {
                throw new ArchiveException(e,
                                           "Invalid descriptor: "
                                           + getPath());
            }
        }
        return stream;
    }

    protected String getArchivePath() {
        return Descriptor.getArchivePath(getType());
    }

    protected boolean isValid() {
        boolean valid = true;

        try {
            validate();
        } catch (ArchiveException e) {
            valid = false;
        }
        return valid;
    }

    protected void validate() throws ArchiveException {
        PathHandle ph = PathHandle.createPathHandle(getPath());

        if (!ph.isFile()) {
            if (isRequired()) {
                throw new ArchiveException("Required descriptor not found: "
                                           + ph.getPath());
            } else if (!ph.isEmpty()) {
                throw new ArchiveException("Invalid descriptor path: "
                                           + ph.getPath());
            }
        }
    }

    public boolean equals(Object comp) {
        Descriptor d = null;
        boolean    equal = false;

        if (comp instanceof Descriptor) {
            d = (Descriptor) comp;
            equal = d.isRequired() == isRequired();
            if (equal) {
                equal = d.getType().equals(getType());
            }
            if (equal) {
                equal = d.getPath().equals(getPath());
            }
        }
        return equal;
    }

}
