/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.archive.wizard;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.archive.ArchiveException;
import org.enhydra.tool.archive.Main;
import org.enhydra.tool.archive.JarPlan;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.wizard.TBWizard;

// JDK
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.beans.*;
import java.util.ResourceBundle;

/**
 * The ArchiveDialog defines a dialog for presenting
 * archive wizards.
 */
public class ArchiveWizard extends TBWizard {

    //
    private final static String             ARCHIVE_TITLE = "Kelp Archive Wizard";

    //
    transient private Icon                  largeIcon = null;
    transient private ArchiveSelectionPanel selectionPanel = null;
    transient private JarPlan               plan = null;
    transient private String                workRoot = null;
    transient private boolean mru = true;

    public ArchiveWizard() {
        super();
        archiveInit();
    }

    /**
     * Get archive
     *
     * @return
     * File object for archive.
     *
     */
    public String getNewArchive() {
        String newArchive = new String();

        if (getPlan() != null) {
            newArchive =
                PathHandle.createPathString(getPlan().getArchivePath());
        }
        return newArchive;
    }

    public JarPlan getPlan() {
        return plan;
    }

    public void setMostRecentlyUsed(boolean b) {
      mru = b;
    }
    public boolean isMostRecentlyUsed() {
      return mru;
    }

    public void setPlan(JarPlan p) throws ArchiveException {
        plan = p;
        ArchiveType type = null;
        type = ArchiveType.createType(plan);
        type.setMostRecentlyUsed(isMostRecentlyUsed());
        getArchiveDeck().setSelection(type);
        getArchiveDeck().setSelectionPageHidden(true);
    }

    /**
     * Set archive
     *
     * @param archive
     * File object for archive.
     *
     */
    public void readArchive(String archive) throws ArchiveException {
        if (getArchiveDeck().getSelection() != null) {
            getArchiveDeck().getSelection().readArchive(archive);
            getArchiveDeck().readPlan();
        }
    }

    public void setWorkingRoot(String r) {
        workRoot = PathHandle.createPathString(r);
        System.setProperty("user.dir", getWorkingRoot());
    }

    public String getWorkingRoot() {
        if (workRoot == null) {
            setWorkingRoot(System.getProperty("user.dir"));
        }
        return workRoot;
    }

    // overrides TBWizard
    public void back() {
        if (getArchiveDeck().getPageIndex() == 1) {
            setLargeIcon(largeIcon);
        }
        super.back();
    }

    // overrides TBWizard
    public void next() {
        ArchiveType type = null;
        if (getArchiveDeck().isSelectionPageShowing()) {
            type = selectionPanel.getSelection();
            try {
                type.setMostRecentlyUsed(isMostRecentlyUsed());
                getArchiveDeck().setSelection(type);
                setLargeIcon(null);
                getArchiveDeck().next();
            } catch (ArchiveException e) {
                JOptionPane.showMessageDialog(getInnerPanel(),
                                              e.getMessage(),
                                              "Archive Wizard Error",
                                              JOptionPane.ERROR_MESSAGE);
                if (Main.debug) {
                    e.printStackTrace();
                }
            }
            refreshButtons();
        } else {
            super.next();
        }
    }

    // override TBWizard
    public void finish() {
        try {
            ArchiveType type = null;

            getArchiveDeck().writePlan();
            type = getArchiveDeck().getSelection();
            type.getPlan().setClassFiltering(true);
            type.build();
            setPlan(type.getPlan());
            JOptionPane.showMessageDialog(getInnerPanel(),
                                          "Created archive:\n"
                                          + getNewArchive(),
                                          "Archive Complete",
                                          JOptionPane.INFORMATION_MESSAGE);
            type.saveDefaults();
            super.finish();
        } catch (ArchiveException e) {
            if (Main.debug) {
                e.printStackTrace();
            }
            JOptionPane.showMessageDialog(getInnerPanel(), e.getMessage(),
                                          "Archive Error",
                                          JOptionPane.ERROR_MESSAGE);
        }
    }

    // implements TBWizard
    public String getTitle() {
        return ARCHIVE_TITLE;
    }

    // implements TBWizard
    protected String getProgressTitle() {
        return "Archive Progress";
    }


    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void archiveInit() {
        ArchiveDeck newDeck = null;
        ArchivePage page = null;

        largeIcon = getLargeIcon();
        newDeck = new ArchiveDeck(this);
        selectionPanel = new ArchiveSelectionPanel();
        selectionPanel.setWizard(this);
        setDeck(newDeck);
        page = new ArchivePage();
        page.setPageTitle(selectionPanel.getPageTitle());
        page.setInstructions(selectionPanel.getInstructions());
        page.add(selectionPanel);
        addWizardPage(page);
    }

    private ArchiveDeck getArchiveDeck() {
        return (ArchiveDeck) getDeck();
    }

}
