/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.configure;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.common.ExtensionFilter;
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.io.File;
import java.util.ResourceBundle;
import java.util.ArrayList;
import java.util.Arrays;

//
public class Main implements Constants {
    static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.configure.Res");    // nores

    //
    private final String  PARAM_JAVA = "-java";          // nores
    private final String  PARAM_ENHYDRA = "-enhydra";    // nores
    private final String  PARAM_APP = "-app";            // nores
    private final String  PARAM_ALL = "-all";            // nores

    //
    private ConfigTool    tool = null;
    private boolean       copyNonTemplates = false;

    public Main() {
        System.out.println("Kelp Configuration Tool " +
           ToolBoxInfo.getToolBoxVersion());
        System.out.println(ToolBoxInfo.getCopyright());
        tool = new ConfigTool();
        tool.setOwner(null);
        tool.setEcho(true);
    }

    public static void main(String[] args) {
        Main main = new Main();

        main.init(args);
    }

    //
    //
    private void init(String[] args) {
        try {
            parseArgs(args);
            tool.configure();
            if (copyNonTemplates) {
                doCopy();
            }
            System.out.println(new String());
            System.out.println(ResUtil.format(res.getString("Enhydra_home_0_"),
                                              tool.getEnhydraRoot()));
            System.out.println(ResUtil.format(res.getString("Java_home_0_"),
                                              tool.getJavaPath()));
            if (tool.getAppPath() != null) {
                System.out.println(ResUtil.format(res.getString("Application_home_0_"),
                                                  tool.getAppPath()));
            }
            System.out.println(new String());
            System.out.println(res.getString("Configuration"));
            System.out.println(new String());
        } catch (ParameterException e) {
            showUsage();
            System.out.println(ResUtil.format(res.getString("Error_0_"),
                                              e.getMessage()));
        } catch (ConfigException e) {
            System.out.println(ResUtil.format(res.getString("Error_0_"),
                                              e.getMessage()));
        }
    }

    private void doCopy() {
        File            sourceDir = null;
        ExtensionFilter copyFilter = null;

        sourceDir = new File(tool.getAppPath() + File.separator + DIR_INPUT);
        copyFilter = new ExtensionFilter();
        copyFilter.addExclusion(".in");
        copyFilter.setExcludeOnly(true);
        recurseCopy(sourceDir, copyFilter);
    }

    private void recurseCopy(File sourceDir, ExtensionFilter copyFilter) {
        File[]     sourceFiles = new File[0];
        PathHandle dest = null;

        sourceFiles = sourceDir.listFiles(copyFilter);
        for (int i = 0; i < sourceFiles.length; i++) {
            if (sourceFiles[i].isDirectory()) {
                recurseCopy(sourceFiles[i], copyFilter);
            } else {
                dest = getDestPath(sourceFiles[i]);
                try {
                    FileUtil.copy(sourceFiles[i], dest.getFile());
                    System.out.println(res.getString("Copying_input_to")
                                       + dest.getPath());
                } catch (ToolException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private PathHandle getDestPath(File inFile) {
        PathHandle   inputPath = null;
        PathHandle   filePath = null;
        PathHandle   rootPath = null;
        PathHandle   passPath = null;
        StringBuffer buf = new StringBuffer();
        String       suffix = new String();

        inputPath = PathHandle.createPathHandle(tool.getAppPath()
                                                + File.separator + DIR_INPUT);
        filePath = PathHandle.createPathHandle(inFile);
        rootPath = PathHandle.createPathHandle(tool.getAppPath()
                                               + File.separator + DIR_OUTPUT);
        buf.append(rootPath.getPath());
        if (inputPath.parentOf(filePath)) {
            suffix =
                filePath.getPath().substring(inputPath.getPath().length());
        } else {
            suffix = filePath.getFile().getName();
        }
        if ((suffix.length() == 0) || (suffix.charAt(0) != '/')) {
            buf.append('/');
        }
        buf.append(suffix);
        passPath = PathHandle.createPathHandle(buf.toString());
        return passPath;
    }

    private String[] extractAllOption(String[] args) {
        boolean   all = false;
        boolean   app = false;
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(args));
        for (int i = 0; i < args.length; i++) {
            if (args[i].equalsIgnoreCase(PARAM_ALL)) {
                all = true;
                list.remove(args[i]);
            }
            if (args[i].equalsIgnoreCase(PARAM_APP)) {
                app = true;
            }
        }
        copyNonTemplates = app && all;
        list.trimToSize();
        args = new String[list.size()];
        args = (String[]) list.toArray(args);
        return args;
    }

    private void parseArgs(String[] args) throws ParameterException {
        args = extractAllOption(args);
        if (args.length == 0) {

            // done
        } else if ((args.length % 2) == 1) {
            throw new ParameterException(res.getString("Wrong_number_of"));
        } else if (args.length > 6) {
            throw new ParameterException(res.getString("Too_many_parameters"));
        } else {
            for (int i = 0; i < args.length; i += 2) {
                if (!isKey(args[i])) {
                    throw new ParameterException(ResUtil.format(res.getString("Invalid_parameter_0_"),
                                                                args[i]));
                }
                int dirIndex = i + 1;

                if (dirIndex < args.length) {
                    if ((!FileUtil.isDirectory(args[dirIndex]))) {
                        throw new ParameterException(ResUtil.format(res.getString("Directory_not_found_0"),
                                                                    args[dirIndex]));
                    } else {
                        initProperty(args[i], args[dirIndex]);
                    }
                }
            }
        }
    }

    private void initProperty(String key, String in) {
        String path = FileUtil.toCanonicalPath(in);

        path = FileUtil.toJavaPath(path);
        if (key.equalsIgnoreCase(PARAM_JAVA)) {
            tool.setJavaPath(path);
        } else if (key.equalsIgnoreCase(PARAM_ENHYDRA)) {
            tool.setEnhydraRoot(path);
        } else if (key.equalsIgnoreCase(PARAM_APP)) {
            tool.setAppPath(path);
        }
    }

    private boolean isKey(String key) {
        return (key.equalsIgnoreCase(PARAM_JAVA)
                || key.equalsIgnoreCase(PARAM_ENHYDRA)
                || key.equalsIgnoreCase(PARAM_APP));
    }

    private void showUsage() {
        System.out.println(res.getString("Usage01"));
        System.out.println(res.getString("Usage02"));
        System.out.println(res.getString("Usage03"));
        System.out.println(new String());
        System.out.println(ResUtil.format(res.getString("Usage04"),
                                          PARAM_APP));
        System.out.println(res.getString("Usage05"));
        System.out.println(new String());
        System.out.println(res.getString("Usage06"));
        System.out.println(res.getString("Usage07"));
        System.out.print(ResUtil.format(res.getString("Usage08"),
                                        PARAM_JAVA));
        System.out.print(ResUtil.format(res.getString("Usage09"),
                                        PARAM_ENHYDRA));
        System.out.print(ResUtil.format(res.getString("Usage10"), PARAM_APP,
                                        PARAM_ALL));
        System.out.println(new String());
        System.out.println(new String());
    }

    private class ParameterException extends ConfigException {
        public ParameterException(String msg) {
            super(msg);
        }

    }
}
